from typing import List, Tuple

from oelint_parser.cls_item import Function
from oelint_parser.cls_stash import Stash

from oelint_adv.cls_rule import Rule


class TaskNoAnonPython(Rule):
    def __init__(self) -> None:
        super().__init__(id='oelint.task.noanonpython',
                         severity='warning',
                         message='Avoid anonymous python functions as they expensive and come with all sorts of side effects')

    def check(self, _file: str, stash: Stash) -> List[Tuple[str, int, str]]:
        res = []
        items = stash.GetItemsFor(
            filename=_file, classifier=Function.CLASSIFIER)
        for i in items:
            if i.IsPython and i.FuncName.strip() in ['', 'anonymous']:
                res += self.finding(i.Origin, i.InFileLine)
        return res



from typing import List, Tuple

from oelint_parser.cls_item import Variable
from oelint_parser.cls_stash import Stash
from oelint_parser.parser import INLINE_BLOCK

from oelint_adv.cls_rule import Rule


class VarSRCUriFirstFile(Rule):
    def __init__(self) -> None:
        super().__init__(id='oelint.vars.srcurifile',
                         severity='warning',
                         message='First item of SRC_URI should not be a file:// fetcher, if multiple fetcher are used')

    def check(self, _file: str, stash: Stash) -> List[Tuple[str, int, str]]:
        res = []
        items: List[Variable] = stash.GetItemsFor(filename=_file, classifier=Variable.CLASSIFIER,
                                                  attribute=Variable.ATTR_VAR, attributeValue='SRC_URI')
        _fetcher = []
        for item in items:
            lines = [y.strip('"') for y in item.get_items() if y]

            for x in lines:
                if x == INLINE_BLOCK:
                    _fetcher.append(('inline', item.InFileLine))
                    continue
                _url = stash.GetScrComponents(x)
                if _url['scheme']:
                    _fetcher.append((_url['scheme'], item.InFileLine))
        if _fetcher:
            if any(x[0] not in ['file', 'inline'] for x in _fetcher) and _fetcher[0][0] == 'file':
                res += self.finding(item.Origin, _fetcher[0][1])
        return res



from typing import List, Tuple

from oelint_parser.cls_item import Variable
from oelint_parser.cls_stash import Stash

from oelint_adv.cls_rule import Rule


class VarLicenseRemoteFile(Rule):
    def __init__(self) -> None:
        super().__init__(id='oelint.var.licenseremotefile',
                         severity='warning',
                         message='License-File should be a remote file')

    def check(self, _file: str, stash: Stash) -> List[Tuple[str, int, str]]:
        res = []
        _items: List[Variable] = stash.GetItemsFor(filename=_file, classifier=Variable.CLASSIFIER,
                                                   attribute=Variable.ATTR_VAR, attributeValue='LIC_FILES_CHKSUM')
        _var_whitelist = ['${WORKDIR}', '${S}', '${B}']
        for i in _items:
            components = stash.GetScrComponents(stash.ExpandTerm(_file, i.VarValueStripped))
            if any(components) and components['scheme'] == 'file':
                _clean = components['src']
                for x in _var_whitelist:
                    _clean = _clean.replace(x, '')
                if '${' in _clean:
                    res += self.finding(i.Origin, i.InFileLine)
        return res
